#!/usr/bin/env node

'use strict';

var join = require('path').join;
var minimist = require('minimist');
var GitAddAndCommit = require(join(__dirname, '../'));

//
// Set things up for minimist
//
var booleans = {
	c: 'caseSensitive',
	h: 'help',
	i: 'interactive',
	s: 'silent',
	S: 'strict',
	v: 'version'
};
var needArgs = {
	e: 'except',
	o: 'only'
};
var CLI_OPTIONS = Object.assign({}, booleans, needArgs);

var argv = minimist(process.argv.slice(2), {
	alias: CLI_OPTIONS,
	boolean: Object.keys(booleans).map(function (key) {
		return booleans[key];
	})
});

//
// Setup options object
//
var options = {};
setupOptions(options);
parseFileTypesToFind(options);
new GitAddAndCommit(argv._.slice(), options);

//
// Helpers
//
function setupOptions(options) {
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = Object.keys(booleans).map(function (key) {
			return booleans[key];
		})[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var option = _step.value;

			options[option] = argv[option];
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}
}

function parseFileTypesToFind(options) {
	var _letterToFileMap = {
		d: 'deleted',
		m: 'modified',
		s: 'staged',
		u: 'untracked'
	};

	// defaults
	var find = {
		deleted: true,
		modified: true,
		staged: true,
		untracked: true
	};

	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = (argv.except && argv.except.split(',') || [])[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			var file = _step2.value;
			// that's a cool line
			delete find[_letterToFileMap[file[0] && file[0].toLowerCase()]]; // another cool line
		}

		// need to delete all find files if `argv.only` is present
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}

	find = argv.only ? {} : find;

	var _iteratorNormalCompletion3 = true;
	var _didIteratorError3 = false;
	var _iteratorError3 = undefined;

	try {
		for (var _iterator3 = (argv.only && argv.only.split(',') || [])[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
			var _file = _step3.value;

			find[_letterToFileMap[_file[0].toLowerCase()]] = true;
		}
	} catch (err) {
		_didIteratorError3 = true;
		_iteratorError3 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion3 && _iterator3.return) {
				_iterator3.return();
			}
		} finally {
			if (_didIteratorError3) {
				throw _iteratorError3;
			}
		}
	}

	options.find = Object.keys(find);
}