'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var join = require('path').join;
var execSync = require('child_process').execSync;
var chalk = require('chalk');
var gitFiles = require('git-files');
var jumper = require('terminal-jumper');
var Interactive = require('./Interactive');
var utils = require('./utils');
var pkg = require('../package.json');

/**
 * Reads a file glob and a commit message, and commits to git.
 * @class
 */

var GitAddAndCommit = function () {
	function GitAddAndCommit(args) {
		var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

		_classCallCheck(this, GitAddAndCommit);

		this.options = options;
		this.args = args;

		if (options.help) {
			this.showHelpScreen();
			return;
		}

		if (options.version) {
			console.log(pkg.version);
			return;
		}

		if (options.interactive) {
			new Interactive(options).run();
			return;
		}

		if (args.length === 0) {
			this.showHelpScreen();
			return;
		}

		this.run();
	}

	/**
  * Finds all files that match the given globs and options given by the user.
  * If there are already staged files, it promps the user to continue. Tries
  * to stage all matched files and commits. If there is an error, it lets the
  * user know and resets all files that were staged successfully.
  */


	_createClass(GitAddAndCommit, [{
		key: 'run',
		value: function run() {
			var _this = this;

			if (this.args.length < 2) {
				console.log('File glob and commit message required. See help screen for correct usage.');
				return;
			}

			this.globs = this.args.slice();
			this.message = this.globs.pop();

			// find all files based on the given globs and options
			var files = utils.getUniqueFilesOfTypes(this.options.find);
			this.matches = utils.matchGlobsAgainstFiles(files, this.globs, {
				caseSensitive: this.options.caseSensitive,
				strict: this.options.strict
			});

			this.checkForStagedFiles().then(function (shouldContinue) {
				if (_this.matches.length === 0) {
					console.log('No matches found.');
					throw new Error('No matches found.');
				}

				_this.matches.forEach(function (file) {
					execSync('git add ' + file);
				});

				execSync('git commit -m "' + _this.message + '"');

				if (_this.options.silent) {
					return;
				}

				console.log('Commit successful.');
			}).catch(function () {
				console.log('Error encountered -- aborting.');

				try {
					_this.matches.forEach(function (file) {
						execSync('git reset ' + file);
					});
				} catch (e) {}
			});
		}

		/**
   * If there are already staged files that don't match any of the given globs,
   * prints them to the screen and prompts the user to continue with the commit.
   */

	}, {
		key: 'checkForStagedFiles',
		value: function checkForStagedFiles() {
			var _this2 = this;

			return new Promise(function (resolve, reject) {
				var extraStagedFiles = gitFiles.staged('relative').filter(function (file) {
					return !_this2.matches.includes(file);
				});

				if (extraStagedFiles.length === 0) {
					resolve();
				} else {
					jumper.block(chalk.bold('There are files already staged for commit but do not match the glob(s) given.'));
					extraStagedFiles.forEach(function (file) {
						return jumper.block('  ' + chalk.green(file));
					});
					jumper.break();

					jumper.block(chalk.bold('Continue? (y/n): '), 'continue');
					jumper.render();
					jumper.jumpTo('continue', -1);

					process.stdin.on('data', function (data) {
						process.stdin.end();
						var answer = data.toString('utf8').trim().toLowerCase();

						// without eval, node will throw a `cannot read property 'includes' of undefined`. Bonkers.
						eval('[\'yes\', \'y\'].includes(answer) ? resolve() : reject()');
					});
				}
			});
		}

		/**
  	 * Shows the help screen.
    */

	}, {
		key: 'showHelpScreen',
		value: function showHelpScreen() {
			var helpFile = join(__dirname, '../help.txt');
			var helpScreen = execSync('cat ' + helpFile).toString('utf8');
			console.log(helpScreen);
		}
	}]);

	return GitAddAndCommit;
}();

module.exports = GitAddAndCommit;